import sys

from krcg import vtes

from . import _utils


def add_parser(parser):
    parser = parser.add_parser("complete", help="card name completion")
    parser.add_argument("-f", "--full", action="store_true", help="display cards text")
    parser.add_argument("name", metavar="NAME", help="parts of the name")
    parser.set_defaults(func=complete)


def complete(args):
    _utils._init()
    completions = vtes.VTES.complete(args.name)
    if not completions:
        sys.stderr.write("No match\n")
        return 1
    print("\n".join(completions))
    return 0



import sys

from krcg import analyzer
from krcg import vtes

from . import _utils


def add_parser(parser):
    parser = parser.add_parser(
        "build", help="build a deck around given card(s), based on the TWDA"
    )
    _utils.add_twda_filters(parser)
    parser.add_argument("cards", metavar="CARD", nargs="*", help="card names or IDs")
    parser.set_defaults(func=build)


def build(args):
    decks = _utils.filter_twda(args)
    try:
        cards = [vtes.VTES[name] for name in args.cards]
    except KeyError as e:
        sys.stderr.write(f"Card not found: {e.args[0]}\n")
        return 1
    print(analyzer.Analyzer(decks).build_deck(*cards).to_txt())
    return 0



import sys

from krcg import twda
from krcg import utils as krcg_utils
from krcg import vtes

from . import _utils


def add_parser(parser):
    parser = parser.add_parser("deck", help="show TWDA decks")
    _utils.add_twda_filters(parser)
    parser.add_argument(
        "-f", "--full", action="store_true", help="display each deck content"
    )
    parser.add_argument(
        "filter",
        metavar="TXT",
        nargs="*",
        help="list of TWDA decks IDs, author names or card names",
    )
    parser.set_defaults(func=deck)


def deck(args):
    _utils._init(with_twda=True)
    filters = set(args.filter)
    joined_args = krcg_utils.normalize(" ".join(args.filter))
    deck_ids = [i for i in args.filter if i in twda.TWDA]
    filters -= set(deck_ids)
    cards = [vtes.VTES[c] for c in args.filter if c in vtes.VTES]
    filters -= set(cards)
    if joined_args in vtes.VTES:
        cards.append(vtes.VTES[joined_args])
        filters.clear()
    authors = [krcg_utils.normalize(a) for a in args.filter]
    authors = [a for a in authors if a in twda.TWDA.by_author]
    filters -= set(authors)
    if joined_args in twda.TWDA.by_author:
        authors.append(joined_args)
        filters.clear()
    if filters:
        sys.stderr.write(
            f"\"{' '.join(filters)}\" did not match a deck #, card or author"
        )
        return 1
    decks = _utils.filter_twda(args)
    if deck_ids or cards or authors:
        decks = [
            d
            for d in decks
            if d.id in deck_ids
            or (cards and all(c in d for c in cards))
            or krcg_utils.normalize(d.player) in authors
            or krcg_utils.normalize(d.author) in authors
        ]
    if len(decks) == 1:
        args.full = True
    if not args.full:
        print(f"-- {len(decks)} decks --")

    for d in sorted(decks, key=lambda a: a.date):
        if args.full:
            print(f"[{d.id:<15}]===================================================")
            print(d.to_txt())
        else:
            print(f"[{d.id}] {d.name}")
    return 0



import argparse

from .subcommands import affinity
from .subcommands import build
from .subcommands import card
from .subcommands import complete
from .subcommands import deck
from .subcommands import format
from .subcommands import search
from .subcommands import seating
from .subcommands import top


parser = argparse.ArgumentParser(prog="krcg", description="VTES tool")
subparsers = parser.add_subparsers(metavar="", title="subcommands", dest="subcommand")
card.add_parser(subparsers)
complete.add_parser(subparsers)
search.add_parser(subparsers)
deck.add_parser(subparsers)
top.add_parser(subparsers)
affinity.add_parser(subparsers)
build.add_parser(subparsers)
format.add_parser(subparsers)
seating.add_parser(subparsers)


def execute(args):
    args = parser.parse_args(args)
    if not args.subcommand:
        parser.print_help()
        return 0
    return args.func(args)



import argparse
import collections
import functools
import itertools
import multiprocessing
import random
import sys

from krcg import seating


def add_parser(parser):
    parser = parser.add_parser(
        "seating",
        help="compute optimal seating",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=(
            """Optimal seating is useful for tournament play.
An optimal seating follows the rules established by the Rules Director, see
https://groups.google.com/g/rec.games.trading-cards.jyhad/c/4YivYLDVYQc/m/CCH-ZBU5UiUJ

The output is a normalised comma-separated list of players, one line per round
Tables of 5 are first, tables of 4, if any, are last.
For example, 1,2,3,4,5,6,7,8,9,10,11,12,13 unambiguously means:
[1, 2, 3, 4, 5], [6, 7, 8, 9], [10, 11, 12, 13]

Use the -v option to display the table structure for each round.
The comma-separated normal form can be used as input of the command
to indicate rounds that have already been played.
This allows for players list modifications during a tournament. For example:

$ krcg seating --played 1,2,3,4,5,6,7,8,9 --remove 4

will output a new seating with the played round left untouched,
and the next 2 rounds without player 4.

Seating for 6, 7 or 11 players required multiple intertwined rounds.
For example, for 6 players to play 2 rounds each, 3 rounds are required
with some players sitting out on each of them:

$ krcg seating --rounds 2 6
2,6,1,4
4,1,5,3
3,5,6,2

This can also be the case when you remove players and come down to 6, 7, or 11 players.
The command will accomodate if they are enough rounds left for intertwined rounds:

$ krcg seating --played 1,2,3,4,5,6,7,8,9 --remove 4 5

Note that when you began to play such intertwined rounds, you cannot modify
the players list in the middle of them. Trying to use the command in this case will
yield unusable seatings listing only the players of the last round.
"""
        ),
    )
    parser.add_argument(
        "players",
        type=int,
        nargs="?",
        metavar="PLAYERS",
        help=("Number of players."),
    )
    parser.add_argument(
        "-r",
        "--rounds",
        type=int,
        default=3,
        help=("Number of rounds"),
    )
    parser.add_argument(
        "-i",
        "--iterations",
        type=int,
        default=80000,
        help="Number of iterations to use (less is faster but may yield worse results)",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Display seating tables and statistics",
    )
    parser.add_argument(
        "--archon",
        action="store_true",
        help="Display Archon-compatible seating lines (empty cell for empty 5th seat)",
    )
    parser.add_argument(
        "-p",
        "--played",
        type=lambda s: [int(x) for x in s.split(",") if x],
        nargs="*",
        metavar="ROUND",
        help="Rounds that have already been played",
    )
    parser.add_argument(
        "--remove",
        type=int,
        nargs="*",
        metavar="PLAYER",
        help="Remove given players",
    )
    parser.add_argument(
        "--add",
        type=int,
        nargs="*",
        metavar="PLAYER",
        help="Add given players",
    )
    parser.set_defaults(func=seat)


POSITIONS = {
    1: "prey",
    2: "grand-prey",
    3: "grand-predator",
    4: "predator",
    5: "cross-table",
}

GROUPS = {
    1: "adjacent",
    2: "non-adjacent",
}


def progression(iterations, step, **kwargs):
    print(f"\t{step / iterations * 100:.0f}%", file=sys.stderr, end="\r")


def seat(options):
    if options.players and options.played:
        print(
            "the [played] and [players] arguments cannot be used both", file=sys.stderr
        )
        return 1
    if not options.players and not options.played:
        print("one of [played] or [players] arguments must be used", file=sys.stderr)
        return 1

    if options.players:
        players = set(range(1, options.players + 1))
        rounds = options.rounds
    else:
        players = set(options.played[-1])
        rounds = options.rounds - len(options.played)

    for player in options.add or []:
        if player in players:
            print(
                f"trying to add {player} but they are already in",
                file=sys.stderr,
            )
        players.add(player)
    for player in options.remove or []:
        if player not in players:
            print(
                f"trying to remove {player} but they absent",
                file=sys.stderr,
            )
            return 1
        players.remove(player)

    try:
        permutations = seating.permutations(len(players), rounds)
    except RuntimeError:
        print(
            "seating cannot be arranged - more rounds or players required",
            file=sys.stderr,
        )
        return 1

    players = list(players)
    if not options.archon:
        random.shuffle(players)
    players = {i: p for i, p in enumerate(players, 1)}
    permutations = (options.played or []) + [
        [players[i] for i in permutation] for permutation in permutations
    ]

    if rounds > 0:
        try:
            cpus = multiprocessing.cpu_count()
        except NotImplementedError:
            cpus = 1
        with multiprocessing.Pool(processes=cpus) as pool:
            results = [
                pool.apply_async(
                    seating.optimise,
                    kwds=dict(
                        permutations=permutations,
                        iterations=options.iterations,
                        callback=functools.partial(progression, options.iterations),
                        fixed=max(1, len(options.played or [])),
                        ignore=set((options.add or []) + (options.remove or [])),
                    ),
                )
                for _ in range(cpus)
            ]
            rounds, score = min((r.get() for r in results), key=lambda x: x[1].total)
            print("", file=sys.stderr, end="")
    else:
        rounds = [seating.Round(p) for p in permutations]
        score = seating.score_rounds(rounds)
    for round_ in rounds:
        delimiter = ","
        if options.archon:
            delimiter = "\t"
            for table in round_:
                if len(table) == 4:
                    table.append("")
        print(delimiter.join(str(p) for p in itertools.chain.from_iterable(round_)))
    if not options.verbose:
        return 0
    print("--------------------------------- details ---------------------------------")
    for i, round_ in enumerate(rounds, 1):
        print(f"Round {i}: {round_}")
    for index, (code, label, _) in enumerate(seating.RULES):
        s = f"{code} {score.rules[index]:6.2f} "
        if score.rules[index]:
            s += f"NOK ({label}): {format_anomalies(score, code)}"
        else:
            s += f" OK ({label})"
        print(s)
    return 0


def format_anomalies(score, code):
    anomalies = getattr(score, code)
    if code in ["R1", "R2", "R4"]:
        return ", ".join(f"{a}-{b}" for a, b in anomalies)
    if code == "R3":
        return partition(score.vps, score.mean_vps)
    if code == "R5":
        return ", ".join(f"{player} twice" for player in anomalies)
    if code == "R6":
        return ", ".join(
            f"{p2} is {p1} {POSITIONS[p]} twice" for p1, p2, p in anomalies
        )
    if code == "R7":
        return ", ".join(f"{player} seats {seat} twice" for player, seat in anomalies)
    if code == "R8":
        return partition(score.transfers, score.mean_transfers)
    if code == "R9":
        return ", ".join(f"{p1} is {p2} {GROUPS[g]} twice" for p1, p2, g in anomalies)
    raise RuntimeError(f"unknown rule {code}")


def partition(anomalies, mean):
    partitions = collections.defaultdict(list)
    for player, value in anomalies:
        partitions[value].append(player)
    return f"mean is {mean:.2f}, " + ", ".join(
        f"{players} {'has' if len(players) < 2 else 'have'} {value}"
        for value, players in sorted(partitions.items())
    )



import sys

from krcg import vtes

from . import _utils


def add_parser(parser):
    parser = parser.add_parser("card", help="show cards")
    parser.add_argument(
        "-i", "--international", action="store_true", help="display translations"
    )
    parser.add_argument(
        "-s", "--short", action="store_true", help="display only card name"
    )
    parser.add_argument(
        "-t", "--text", action="store_true", help="display card text only (no rulings)"
    )
    parser.add_argument(
        "-l", "--links", action="store_true", help="display ruling links"
    )
    parser.add_argument("cards", metavar="CARD", nargs="*", help="card names or IDs")
    parser.set_defaults(func=card)


def card(args):
    """Display cards, their text and rulings"""
    _utils._init()
    index = 0
    cards = args.cards
    if not cards and not sys.stdin.isatty():
        cards = sys.stdin.read().splitlines()
    try:
        for name in cards:
            _display_card(args, name, index)
            index += 1
        return 0
    except KeyError:
        if index == 0:
            try:
                _display_card(args, " ".join(cards))
                return 0
            except KeyError:
                pass
    sys.stderr.write("Card not found\n")
    return 1


def _display_card(args, name: str, index: int = 0) -> None:
    if not args.short and index > 0:
        print()
    try:
        name = int(name)
    except ValueError:
        pass
    card = vtes.VTES[name]
    print(card.name)
    if args.international:
        for lang, translation in card.i18n_variants("name"):
            print(f"  {lang[:2]} -- {translation}")
    if args.short:
        return
    print(_card_text(card))
    if args.international:
        for lang, translation in card.i18n_variants("card_text"):
            print(f"\n-- {lang[:2]}\n{translation}")
    if args.text or not card.rulings.get("text"):
        return
    print(_card_rulings(args, card))


def _card_text(card) -> str:
    """Full text of a card (id, title, traits, costs, ...) for display purposes"""
    text = "[{}]".format("/".join(card.types))
    if card.clans:
        text += "[{}]".format("/".join(card.clans))
    if card.pool_cost:
        text += "[{}P]".format(card.pool_cost)
    if card.blood_cost:
        text += "[{}B]".format(card.blood_cost)
    if card.conviction_cost:
        text += "[{}C]".format(card.conviction_cost)
    if card.capacity:
        text += "[{}]".format(card.capacity)
    if card.group:
        text += "(g.{})".format(card.group)
    if card.burn_option:
        text += "(Burn Option)"
    if card.banned:
        text += " -- BANNED in " + card["Banned"]
    text += " -- (#{})".format(card.id)
    if card.crypt and card.disciplines:
        text += "\n{}".format(" ".join(card.disciplines) or "-- No discipline")
    text += "\n{}".format(card.card_text)
    return text


def _card_rulings(args, card):
    """Text of a card's rulings"""
    text = "\n-- Rulings\n"
    for ruling in card.rulings["text"]:
        text += ruling + "\n"
    if args.links:
        for ref, link in card.rulings["links"].items():
            text += f"{ref}: {link}\n"
    return text[:-1]



#!/usr/bin/env python3
import logging
import sys

from . import parser


def main():
    logging.basicConfig(format="[%(levelname)7s] %(message)s")
    exit(parser.execute(sys.argv[1:]))


if __name__ == "__main__":
    main()



import sys

from krcg import analyzer
from krcg import vtes

from . import _utils


def add_parser(parser):
    _utils._init()
    parser = parser.add_parser(
        "affinity", help="display cards affinity (most played together)"
    )
    parser.add_argument(
        "--min",
        type=int,
        default=25,
        help="Minimum affinity score to display (0 to 100, default 25)",
    )
    _utils.add_twda_filters(parser)
    parser.add_argument("cards", metavar="CARD", nargs="*", help="card names or IDs")
    parser.set_defaults(func=affinity)


def affinity(args):
    decks = _utils.filter_twda(args)
    try:
        cards = [vtes.VTES[name] for name in args.cards]
    except KeyError as e:
        sys.stderr.write(f"Card not found: {e.args[0]}\n")
        return 1
    A = analyzer.Analyzer(decks)
    A.refresh(*cards, similarity=1)
    if len(A.examples) < 4:
        print("Too few example in TWDA.")
        if len(A.examples) > 0:
            print(
                "To see them:\n\tkrcg deck "
                + " ".join('"' + card.name + '"' for card in cards)
            )
        return 0
    # do not include spoilers if affinity is within 50% of natural occurence
    candidates = A.candidates(*cards, spoiler_multiplier=1.5)
    for card, score in candidates:
        score = round(score * 100 / len(cards))
        if args.min > score:
            break
        print(
            f"{card.name:<30} (in {score:.0f}% of decks, typically "
            f"{_utils.typical_copies(A, card)})"
        )
    return 0



import argparse
import json
import sys

from krcg import deck


def add_parser(parser):
    parser = parser.add_parser("format", help="format a decklist")
    parser.add_argument(
        "-f",
        "--format",
        help="Format",
        required=True,
        type=str.lower,
        choices=["jol", "twd", "lackey", "json"],
    )
    parser.add_argument(
        "infile",
        nargs="?",
        type=argparse.FileType("r"),
        default=sys.stdin,
        help="Input file. If not provided, read from standard input (stdin)",
    )
    parser.set_defaults(func=format)


def format(args):
    d = None
    try:
        d = deck.Deck.from_txt(args.infile)
    except Exception as e:
        sys.stderr.write(f"Failed to parse decklist: {e}")
    if args.format == "json":
        json.dump(d.to_json(), sys.stdout, ensure_ascii=False, indent=2)
    else:
        print(d.to_txt(format=args.format))



import argparse
import arrow
import marshal
import math
import tempfile
import os
import sys

from krcg import twda
from krcg import vtes

VTES_FILE = os.path.join(tempfile.gettempdir(), "krcg-vtes.pyc")
TWDA_FILE = os.path.join(tempfile.gettempdir(), "krcg-twda.pyc")


def _init(with_twda=False):
    try:
        if not vtes.VTES:
            vtes.VTES.from_json(marshal.load(open(VTES_FILE, "rb")))
        if with_twda and not twda.TWDA:
            twda.TWDA.from_json(marshal.load(open(TWDA_FILE, "rb")))
    except FileNotFoundError:
        try:
            if not vtes.VTES:
                vtes.VTES.load()
            marshal.dump(vtes.VTES.to_json(), open(VTES_FILE, "wb"))
            if with_twda:
                # if TWDA existed but VTES was not loaded, load TWDA anew
                twda.TWDA.load()
                marshal.dump(twda.TWDA.to_json(), open(TWDA_FILE, "wb"))
        except:  # noqa: E722
            sys.stderr.write("Fail to initialize - check your Internet connection.\n")
            raise


class NargsChoice(argparse.Action):
    """Choices with nargs +/*: this is a known issue for argparse
    cf. https://bugs.python.org/issue9625
    """

    CASE_SENSITIVE = False

    def get_choices(self):
        ...

    def __call__(self, parser, namespace, values, option_string=None):
        choices = self.get_choices()
        if not self.CASE_SENSITIVE:
            values = [v.lower() for v in values]
            choices = {c.lower() for c in choices}
        if values:
            for value in values:
                if value not in choices:
                    raise argparse.ArgumentError(
                        self,
                        f"invalid choice: {value} (choose from: "
                        f"{', '.join(self.get_choices())})",
                    )
        setattr(namespace, self.dest, values)


def add_twda_filters(parser):
    parser.add_argument(
        "--from",
        type=lambda s: arrow.get(s).date(),
        dest="date_from",
        help="only consider decks from that date on",
    )
    parser.add_argument(
        "--to",
        type=lambda s: arrow.get(s).date(),
        dest="date_to",
        help="only consider decks up to that date",
    )
    parser.add_argument(
        "--players",
        type=int,
        default=0,
        help="only consider decks that won against at least that many players",
    )


def filter_twda(args):
    _init(with_twda=True)
    decks = list(twda.TWDA.values())
    if args.date_from:
        decks = [d for d in decks if d.date >= args.date_from]
    if args.date_to:
        decks = [d for d in decks if d.date < args.date_to]
    if args.players:
        decks = [d for d in decks if d.players_count >= args.players]
    return decks


class DisciplineChoice(NargsChoice):
    CASE_SENSITIVE = True

    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["discipline"]


class ClanChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["clan"]

    # ALIASES = config.CLANS_AKA


class TypeChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["type"]


class TraitChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["trait"]


class GroupChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["group"]


class BonusChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["bonus"]


class CapacityChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["capacity"]


class SectChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["sect"]


class TitleChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["title"]


class CityChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["city"]


class SetChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["set"]


class RarityChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["rarity"]


class PreconChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["precon"]


class ArtistChoice(NargsChoice):
    @staticmethod
    def get_choices():
        return vtes.VTES.search_dimensions["artist"]


def add_card_filters(parser):
    parser.add_argument(
        "-d",
        "--discipline",
        action=DisciplineChoice,
        metavar="DISCIPLINE",
        nargs="+",
        help="Filter by discipline ({})".format(
            ", ".join(DisciplineChoice.get_choices())
        ),
    )
    parser.add_argument(
        "-c",
        "--clan",
        action=ClanChoice,
        metavar="CLAN",
        nargs="+",
        help="Filter by clan ({})".format(", ".join(ClanChoice.get_choices())),
    )
    parser.add_argument(
        "-t",
        "--type",
        action=TypeChoice,
        metavar="TYPE",
        nargs="+",
        help="Filter by type ({})".format(", ".join(TypeChoice.get_choices())),
    )
    parser.add_argument(
        "-g",
        "--group",
        action=GroupChoice,
        metavar="GROUP",
        nargs="+",
        help="Filter by group ({})".format(
            ", ".join(map(str, GroupChoice.get_choices()))
        ),
    )
    parser.add_argument(
        "-e",
        "--exclude-type",
        action=TypeChoice,
        metavar="TYPE",
        nargs="+",
        help="Exclude given types ({})".format(", ".join(TypeChoice.get_choices())),
    )
    parser.add_argument(
        "-b",
        "--bonus",
        action=BonusChoice,
        metavar="BONUS",
        nargs="+",
        help="Filter by bonus ({})".format(", ".join(BonusChoice.get_choices())),
    )
    parser.add_argument(
        "--text",
        metavar="TEXT",
        nargs="+",
        help="Filter by text (including name and flavor text)",
    )
    parser.add_argument(
        "--trait",
        action=TraitChoice,
        metavar="TRAIT",
        nargs="+",
        help="Filter by trait ({})".format(", ".join(TraitChoice.get_choices())),
    )
    parser.add_argument(
        "--capacity",
        type=int,
        action=CapacityChoice,
        metavar="CAPACITY",
        nargs="+",
        help="Filter by capacity ({})".format(
            ", ".join(map(str, CapacityChoice.get_choices()))
        ),
    )
    parser.add_argument(
        "--set",
        action=SetChoice,
        metavar="SET",
        nargs="+",
        help="Filter by set",
    )
    parser.add_argument(
        "--sect",
        action=SectChoice,
        metavar="SECT",
        nargs="+",
        help="Filter by sect ({})".format(", ".join(SectChoice.get_choices())),
    )
    parser.add_argument(
        "--title",
        action=TitleChoice,
        metavar="TITLE",
        nargs="+",
        help="Filter by title ({})".format(", ".join(TitleChoice.get_choices())),
    )
    parser.add_argument(
        "--city",
        action=CityChoice,
        metavar="CITY",
        nargs="+",
        help="Filter by city",
    )
    parser.add_argument(
        "--rarity",
        action=RarityChoice,
        metavar="RARITY",
        nargs="+",
        help="Filter by rarity ({})".format(", ".join(RarityChoice.get_choices())),
    )
    parser.add_argument(
        "--precon",
        action=PreconChoice,
        metavar="PRECON",
        nargs="+",
        help="Filter by preconstructed starter",
    )
    parser.add_argument(
        "--artist",
        action=ArtistChoice,
        metavar="ARTIST",
        nargs="+",
        help="Filter by artist",
    )


def filter_cards(args):
    _init()
    args = {
        k: v
        for k, v in vars(args).items()
        if k
        in {
            "discipline",
            "clan",
            "type",
            "group",
            "exclude-type",
            "bonus",
            "text",
            "trait",
            "capacity",
            "set",
            "sect",
            "title",
            "city",
            "rarity",
            "precon",
            "artist",
        }
    }
    exclude_type = args.pop("exclude_type", None)
    if exclude_type:
        args["type"] = list(
            args.get("type", set())
            | (set(TypeChoice.get_choices()) - set(exclude_type))
        )
    args["text"] = " ".join(args.pop("text") or [])
    args = {k: v for k, v in args.items() if v}
    return vtes.VTES.search(**args)


def typical_copies(A, card):
    deviation = math.sqrt(A.variance[card])
    min_copies = max(1, round(A.average[card] - deviation))
    max_copies = max(1, round(A.average[card] + deviation))
    if min_copies == max_copies:
        ret = f"{min_copies}"
    else:
        ret = f"{min_copies}-{max_copies}"
    if max_copies > 1:
        ret += " copies"
    else:
        ret += " copy"
    return ret



import sys

from krcg import analyzer

from . import _utils


def add_parser(parser):
    _utils._init()
    parser = parser.add_parser("top", help="display top cards (most played)")
    parser.add_argument(
        "-n",
        "--number",
        type=int,
        default=10,
        help="Number of cards to print (default 10)",
    )
    _utils.add_card_filters(parser)
    _utils.add_twda_filters(parser)
    parser.set_defaults(func=top)


def top(args):
    candidates = _utils.filter_cards(args)
    if not candidates:
        sys.stderr.write("No card match\n")
        return 1
    decks = _utils.filter_twda(args)
    A = analyzer.Analyzer(decks)
    A.refresh(condition=lambda c: c in candidates)
    for card, count in A.played.most_common()[: args.number]:
        print(
            f"{card.name:<30} (played in {count} decks, typically "
            f"{_utils.typical_copies(A, card)})"
        )
    return 0



import sys


from . import _utils


def add_parser(parser):
    _utils._init()
    parser = parser.add_parser("search", help="search card")
    parser.add_argument(
        "-n",
        "--number",
        type=int,
        default=10,
        help="Number of cards to print (default 10)",
    )
    _utils.add_card_filters(parser)
    parser.set_defaults(func=search)


def search(args):
    results = _utils.filter_cards(args)
    if not results:
        sys.stderr.write("No match\n")
        return 1
    results = sorted(c.name for c in results)
    if args.number and args.number < len(results):
        full = len(results)
        results = results[: args.number]
        results.append(
            f"... {full - args.number} more results, use -n {full} to display them."
        )
    print("\n".join(results))
    return 0
